#!/usr/bin/env python
"""
Utility routines to generate density matrices
"""
from math import sqrt
from . import one, sirifc
from .util import full, blocked

def h1diag(nisht, nasht, filename='AOONEINT'):
    """Generate density from diagonalized one-electron Hamiltonian"""
    h1 = one.read('ONEHAMIL', filename).unpack().unblock()
    S = one.read('OVERLAP', filename).unpack().unblock()
    return fdiag(h1, S, nisht, nasht)

def fdiag(F, S, nisht, nasht):
    """Generate density from diagonalizing input Fock matrix"""
    C = cmo(F, S)
    return C2D(C, nisht, nasht)

def cmo(F, S, filename='AOONEINT'):
    """Return MO coefficients from diagonalization of a provided Fock matrix F,
    an optional overlap matrix S. The returned MOs are normalized and ordered 
    with respect to eigenvalue"""
    nbast = F.shape[0]
    e, V = (F/S).eigvec()
    C = full.matrix((nbast, nbast))
    N2 = (V.T*S*V).diagonal()
    for i in range(nbast):
        Ni = 1.0 / sqrt(N2[i])
        C[:, i] = V[:, i]*Ni
        # Set the phase such that the largest component is positive
        Cmx = 0.
        rephase = False
        for j in range(nbast):
            if abs(C[j, i]) > Cmx:
                Cmx = abs(C[j, i])
                rephase = (C[j, i] < 0)
        if rephase:
            C[:, i] *= -1
                
    #
    # Finish off with Gram Schmidt because degenerate orbitals are not
    # properly orthonormalized
    #
    C = C.GS(S)
    #
    #
    return C

def C2D(C, nisht, nasht):
    """Given orbitals and occupancy inactive/active 
    return inactive/active densities"""
    Ci = C[:, :nisht]
    Ca = C[:, nisht:nisht+nasht]
    Di = 2*Ci*Ci.T
    Da = Ca*Ca.T
    return Di, Da

def C2Dab(C, na, nb):
    """Given orbitals and occupancy inactive/active 
    return inactive/active densities"""
    Ca = C[:, :na]
    Cb = C[:, :nb]
    Da = Ca*Ca.T
    Db = Cb*Cb.T
    return Da, Db

def C1D(C, n):
    """Given orbitals C and number of occupied n return density C*C.T"""
    Ci = C[:, :n]
    Di = Ci*Ci.T
    return Di

def ifc(filename='SIRIFC', ifc_=None):
    """
    Return inactive/active densities from orbitals on interface file SIRIFC
    """
    if ifc_ is None: 
        ifc_ = sirifc.sirifc(filename)
    Di = blocked.BlockDiagonalMatrix(ifc_.nbas, ifc_.nbas)
    for  isym in range(ifc_.nsym):
        if ifc_.nish[isym]:
            Ci = ifc_.cmo.subblock[isym]
            Cocci = Ci[:, :ifc_.nish[isym]]
            Di.subblock[isym] = 2*Cocci*Cocci.T
    Di = Di.unblock()
    if ifc_.nasht:
        cmoa = blocked.BlockDiagonalMatrix(ifc_.nbas, ifc_.nash)
        for isym in range(ifc_.nsym):
            if ifc_.nash[isym]:
                nishi = ifc_.nish[isym]
                nocci = ifc_.nish[isym] + ifc_.nash[isym]
                cmoa.subblock[isym] = ifc_.cmo.subblock[isym][:, nishi:nocci]
        cmoa = cmoa.unblock()
        Dv = cmoa*ifc_.dv.unpack()*cmoa.T
    else:
        Dv = full.matrix((ifc_.nbast, ifc_.nbast))
    return Di, Dv

def Dab(*args, **kwargs):
    Di, Dv = ifc(*args, **kwargs)
    Db = 0.5*Di 
    Da = Db + Dv
    return (Da, Db)

if __name__ == "__main__":
    pass



import unittest
import sys
import numpy
from ..core import output_beta, header
from ..daltools.util import full


class NewTest(unittest.TestCase):

    def setUp(self):
        numpy.random.seed(0)
        self.beta = full.matrix((3, 6)).random()
        self.dipole = full.matrix(3).random()

    def tearDown(self):
        pass

    def test_header(self):
        if not hasattr(sys.stdout, "getvalue"):
            self.fail("Run in buffered mode")
        header('yo')
        print_output = sys.stdout.getvalue().strip()
        ref_output = """\
--
yo
--"""
        self.assertEqual(print_output, ref_output)

    def test_output_beta(self):
        if not hasattr(sys.stdout, "getvalue"):
            self.fail("Run in buffered mode")
        output_beta(self.beta)
        print_output = sys.stdout.getvalue().strip()
        ref_output="""\
Hyperpolarizability
beta(:, xx xy xz yy yz zz)
--------------------------
beta(x, *)     0.548814    0.715189    0.602763    0.544883    0.423655    0.645894
beta(y, *)     0.437587    0.891773    0.963663    0.383442    0.791725    0.528895
beta(z, *)     0.568045    0.925597    0.071036    0.087129    0.020218    0.832620
beta(:, kk)    1.739591    1.349924    1.487794"""
        self.assertEqual(print_output, ref_output)

    def test_output_beta_d(self):
        if not hasattr(sys.stdout, "getvalue"):
            self.fail("Run in buffered mode")
        output_beta(self.beta, self.dipole)
        print_output = sys.stdout.getvalue().strip()
        ref_output="""\
Hyperpolarizability
beta(:, xx xy xz yy yz zz)
--------------------------
beta(x, *)     0.548814    0.715189    0.602763    0.544883    0.423655    0.645894
beta(y, *)     0.437587    0.891773    0.963663    0.383442    0.791725    0.528895
beta(z, *)     0.568045    0.925597    0.071036    0.087129    0.020218    0.832620
beta(:, kk)    1.739591    1.349924    1.487794
beta//dip      0.523123"""
        self.assertEqual(print_output, ref_output)

if __name__ == "__main__":
    unittest.main()




import numpy

class BlockDiagonalMatrix(object):
    """ Blocked matrix class based on lists of full matrices"""

    def __init__(self,nrow,ncol):
        """ Constructur of the class.""" 

        assert ( len(nrow) == len(ncol) )
        from . import full
        self.nblocks=len(nrow)
        self.nrow=nrow
        self.ncol=ncol
        self.subblock=[]
        self.irow=[]
        self.icol=[]
        for i in range(self.nblocks):
            self.subblock.append(full.matrix((nrow[i],ncol[i])))
            self.irow.append(sum(self.nrow[:i]))
            self.icol.append(sum(self.ncol[:i]))
        self.irow = tuple(self.irow)
        self.icol = tuple(self.icol)

    def __str__(self):
        """ Formatted output based on full matrix class """

        retstr=""
        for i in range(self.nblocks):
            if (self.nrow[i]*self.ncol[i]):
                retstr+="\nBlock %d\n"%(i+1) + str(self.subblock[i])
        return retstr

    def __getitem__(self,n):
        """ Index argument returns subblock
        Example
        >>> M = BlockDiagonalMatrix([2], [2])
        >>> print M[0]
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
        <BLANKLINE>
        """
        return self.subblock[n]

    @staticmethod
    def init(*arr):
        from .full import init
        matrices = tuple(init(a) for a in arr)
        for m in matrices:
            assert len(m.shape) == 2, "blocked only for two dimensions"
        rdim = tuple(m.shape[0] for m in matrices)
        cdim = tuple(m.shape[1] for m in matrices)
        new = BlockDiagonalMatrix(rdim, cdim)
        for i, a in enumerate(arr):
            new.subblock[i][:, :] = matrices[i]
        return new

    @staticmethod
    def init_from_array(arr, rdim, cdim):
        from . import full
        assert numpy.dot(rdim, cdim) == len(arr)
        new = BlockDiagonalMatrix(rdim, cdim)
        start = 0
        end = 0
        for i, block in enumerate(new.subblock):
            end += rdim[i]*cdim[i]
            block[:, :] = full.init(arr[start: end]).reshape(block.shape, order='F')
            start += rdim[i]*cdim[i]
        return new

    @property
    def size(self):
        return sum(b.size for b in self.subblock)

    def ravel(self, **kwargs):
        from . import full
        linear = full.matrix(self.size)
        start = 0
        end = 0
        for s in self.subblock:
            end += s.size
            linear[start: end] = s.ravel(**kwargs)
            start += s.size
        return linear
        

    def random(self):
        """ Fill matrix subblocks with random numbers
        Example
        >>> M = BlockDiagonalMatrix([2], [2]).random()
        >>> assert M.subblock[0][0,0] < 1 and M.subblock[0][0,0] > 0
        >>> assert M.subblock[0][1,0] < 1 and M.subblock[0][1,0] > 0
        >>> assert M.subblock[0][0,1] < 1 and M.subblock[0][0,1] > 0
        >>> assert M.subblock[0][1,1] < 1 and M.subblock[0][1,1] > 0
        """
        for i in range(self.nblocks):
            self.subblock[i].random()
        return self

    def __mul__(self,other):
        """Multiplication blockwise """
        
        new=BlockDiagonalMatrix(self.nrow,other.ncol)
        for i in range(self.nblocks):
           if self.nrow[i]:
              new.subblock[i]=self.subblock[i]*other.subblock[i]
        return new

    def __rmul__(self,other):
        """Scalar multiplication"""

        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
           if self.nrow[i]:
               new.subblock[i]=other*self.subblock[i]
        return new

    def __add__(self,other):
        """Addition blockwise"""

        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
           if self.nrow[i]:
               new.subblock[i]=self.subblock[i]+other.subblock[i]
        return new

    def __sub__(self,other):
        """Subtraction blockwize"""

        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
            if self.nrow[i]:
                new.subblock[i]=self.subblock[i]-other.subblock[i]
        return new

    def __neg__(self):
        """Negation blockwise"""

        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
            if self.nrow[i]:
                new.subblock[i]=-self.subblock[i]
        return new

    def __truediv__(self,other):
        "Solve linear equations blockwise"""

        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
            if self.nrow[i]:
                if isinstance(other,self.__class__):
                    new.subblock[i]=self.subblock[i]/other.subblock[i]
                else:
                    new.subblock[i]=self.subblock[i]/other
        return new

    def __div__(self, other):
        return self.__truediv__(other)

    def __rtruediv__(self,other):
        """Inversion """
        return unit(self.nrow,other)/self

    def __rdiv__(self, other):
        return self.__rtruediv__(other)

    def pack(self):
        for i in range(self.nblocks):
            assert ( self.nrow[i] == self.ncol[i] )
        new=triangular(self.nrow)
        for i in range(self.nblocks):
            new.subblock[i]=self.subblock[i].pack()
        return new

    def unblock(self):
        from . import full
        nrows=sum(self.nrow)
        ncols=sum(self.ncol)
        new=full.matrix((nrows,ncols))
        for i in range(self.nblocks):
            new[self.irow[i]:self.irow[i]+self.nrow[i],self.icol[i]:self.icol[i]+self.ncol[i]]=self.subblock[i]
        return new

    def T(self):
        """Transpose
        Example:
        >>> M=BlockDiagonalMatrix([2],[2]); M.subblock[0][0,1]=1; M.subblock[0][1,0]=2
        >>> print M, M.T()
        <BLANKLINE>
        Block 1
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      0.00000000    1.00000000
               2      2.00000000    0.00000000
        <BLANKLINE>
        Block 1
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      0.00000000    2.00000000
               2      1.00000000    0.00000000
        <BLANKLINE>
        """
        new=BlockDiagonalMatrix(self.ncol,self.nrow)
        for i in range(self.nblocks):
            new.subblock[i]=self.subblock[i].T
        return new

    def sqrt(self):
        new=BlockDiagonalMatrix(self.ncol,self.nrow)
        for i in range(self.nblocks):
            new.subblock[i]=self.subblock[i].sqrt()
        return new

    def invsqrt(self):
        new=BlockDiagonalMatrix(self.ncol,self.nrow)
        for i in range(self.nblocks):
            new.subblock[i]=self.subblock[i].invsqrt()
        return new
        

    def func(self,f):
         """ Blockwise function of matrix"""
         new=BlockDiagonalMatrix(self.ncol,self.nrow)
         for i in range(self.nblocks):
             new.subblock[i]=self.subblock[i].func(f)
         return new

    def tr(self):
        """Sum blockwise traces
        """

        sum=0
        for i in range(self.nblocks):
            if self.nrow[i]:
                sum+=self.subblock[i].tr()
        return sum

    def eigvec(self):
        u=BlockDiagonalMatrix(self.nrow,self.nblocks*[1])
        v=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
           u.subblock[i],v.subblock[i]=self.subblock[i].eigvec()
        return u,v

    def qr(self):
        q=BlockDiagonalMatrix(self.nrow,self.nrow)
        r=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
          q.subblock[i],r.subblock[i]=self.subblock[i].qr()
        return q,r

    def GS(self,S):
        new=BlockDiagonalMatrix(self.nrow,self.ncol)
        for i in range(self.nblocks):
           new.subblock[i]=self.subblock[i].GS(S.subblock[i])
        return new

    def get_columns(self, columns_per_symmetry):
        new = BlockDiagonalMatrix(self.nrow, columns_per_symmetry)
        for oldblock, newblock, cols in zip(self, new, columns_per_symmetry):
            if cols:
                newblock[:, :] = oldblock[:, :cols]
        return new
         
      
def unit(nbl,factor=1):
   from .full import unit
   new=BlockDiagonalMatrix(nbl,nbl)
   for i in range(len(nbl)):
      if nbl[i]:
         new.subblock[i]=unit(nbl[i],factor)
   return new

class triangular(object):

   def __init__(self,dim):
      from . import full
      self.nblocks=len(dim)
      self.dim=dim
      self.subblock=[]
      for i in range(self.nblocks):
         self.subblock.append(full.triangular((dim[i],dim[i])))

   def __str__(self):
      retstr=""
      for i in range(self.nblocks):
         if (self.dim[i]):
            retstr+="\nBlock %d\n"%(i+1) + str(self.subblock[i])
      return retstr

   def random(self):
      for i in range(self.nblocks):
         self.subblock[i].random()

   def __add__(self,other):
      new=triangular(self.dim)
      for i in range(self.nblocks):
         new.subblock[i]=self.subblock[i]+other.subblock[i]
      return new

   def __sub__(self,other):
      new=triangular(self.dim)
      for i in range(self.nblocks):
         new.subblock[i]=self.subblock[i]-other.subblock[i]
      return new

   def unpack(self):
      new=BlockDiagonalMatrix(self.dim,self.dim)
      for i in range(self.nblocks):
         new.subblock[i]=self.subblock[i].unpack()
      return new

   def unblock(self):
      return self.unpack().unblock().pack()

   @staticmethod
   def init(blocks):
        from . import full
        triangular_matrices = [full.triangular.init(block) for block in blocks]
        dimensions = [tmat.dim for tmat in triangular_matrices]
        new = triangular(dimensions)
        new.subblock = triangular_matrices
        return new
        



import unittest
import os
import numpy as np
from .. import rspvec, sirifc

class TestOpenRspVec(unittest.TestCase):

    def setUp(self):
        n, _ = os.path.splitext(__file__)
        self.tmpdir = n + ".d"
        self.RSPVEC = os.path.join(self.tmpdir, 'RSPVEC')
        self.SIRIFC = os.path.join(self.tmpdir, 'SIRIFC')
        self.ifc = sirifc.sirifc(name=self.SIRIFC)

    def test_read(self):
        Nx = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        this = Nx[5]
        ref = -2.34009730
        self.assertAlmostEqual(this, ref)

    def test_tomat(self):
        Nx = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        kx = rspvec.tomat(Nx, self.ifc, tmpdir=self.tmpdir)
        this = kx[5, 7]
        ref = -2.34009730
        self.assertAlmostEqual(this, ref)


    def test_tovec(self):
        ref = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        kx = rspvec.tomat(ref, self.ifc, tmpdir=self.tmpdir)
        this = rspvec.tovec(kx, self.ifc, tmpdir=self.tmpdir)
        np.testing.assert_almost_equal(this, ref)

    def test_jwop(self):
        this = list(rspvec.jwop(self.ifc))
        ref = [ (i, j) for i in range(7) for j in range(7, 8)] + \
              [ (i, j) for i in range(7) for j in range(8, 11)] + \
              [ (i, j) for i in range(7,8) for j in range(8, 11)]
        np.testing.assert_almost_equal(this, ref)

if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""Module for reading response vector data from RSPVEC"""

THRESHOLD = 1e-5

import pdb
import numpy as np
import time
from .util import full, unformatted

class RspVecError(Exception): pass

def read(*args, **kwargs):
    """Read response vector given property"""
    if 'freqs' in kwargs:
        #linear response/one frequency
        bfreqs = kwargs.get('freqs')
    else:
        #non-linear response/two frequencies
        bfreqs = kwargs.get('bfreqs', (0.0,))
    cfreqs = kwargs.get('cfreqs', (0.0,))
    propfile = kwargs.get('propfile', 'RSPVEC')
    lr_vecs = kwargs.get('lr_vecs', False)

    rspvec = unformatted.FortranBinary(propfile)
    vecs = {}

    
    for rec in rspvec:
        for lab in args:
            try:
                lab1 = bytes(lab.ljust(16), 'utf-8')
            except TypeError:
                lab1 = bytes(lab.ljust(16))
            #alternative label with permuted labels
            lab2 = lab1[8:] + lab1[:8]
            if lab1 in rec or lab2 in rec:
                if lab1 in rec:
                    rec.read(16,'c')
                    bfreq, cfreq = rec.read(2, 'd')
                elif lab2 in rec:
                    rec.read(16,'c')
                    cfreq, bfreq = rec.read(2, 'd')
                if bfreq in bfreqs and cfreq in cfreqs:
                    rspvec.next()
                    kzyvar = rspvec.reclen // 8
                    buffer_ = rspvec.readbuf(kzyvar,'d')
                    vecs[(lab, bfreq, cfreq)] = \
                       np.array(buffer_).view(full.matrix)
                    vecs[(lab1, bfreq, bfreq)] = vecs[(lab, bfreq, cfreq)]
                    vecs[(lab2, cfreq, bfreq)] = vecs[(lab, bfreq, cfreq)]


    # now check that all required vectors are saved
    for l in args:
        for b in bfreqs:
            for c in cfreqs:
                if (l, b, c) not in vecs:
                    raise RspVecError(
                        "Response vector N(%s,%g,%g) not found on file %s" %
                        (l, b, c, propfile)
                    )

    # complement dict with lr pointers
    if cfreqs == (0.0,):
        vecs.update({(l,b):vecs[(l,b,0.0)]  for l in args for b in bfreqs})
        if bfreqs == (0.0,):
            vecs.update({l:vecs[(l,0.0,0.0)]  for l in args})
    if lr_vecs:
        keys = ((a, w) for a in args for w in bfreqs)
        return {k: vecs[k] for k in keys}
    else:
        return vecs
    #return [[vecs[(l, b, c)] for l in args] for b in bfreqs for c in cfreqs]

def readall(property_label, propfile="RSPVEC"):
    """Read response all vectors given property"""
    import time, numpy
    _rspvec = unformatted.FortranBinary(propfile)
    found=[]
    while _rspvec.find(property_label) is not None:
        veclabs = _rspvec.rec.read(16,'c')
        vfreq = _rspvec.rec.read(1, 'd')[0]
        _rspvec.next()
        kzyvar = _rspvec.reclen // 8
        buffer_ = _rspvec.readbuf(kzyvar,'d')
        mat = numpy.array(buffer_).view(full.matrix)
        found.append((mat, vfreq))
    return found

def tomat(N, ifc, tmpdir='/tmp'):
    """Vector to matrix"""
    import os
    norbt = ifc.norbt
    new = full.matrix((norbt, norbt))
    lwop = len(N)/2
    ij = 0
    for i, j in jwop(ifc):
        new[i, j] = N[ij]
        new[j, i] = N[ij+lwop]
        ij += 1
    return new

def tovec(mat, ifc, tmpdir='/tmp'):
    """Vector to matrix"""
    import os
    lwop = ifc.nisht*ifc.nasht + ifc.nocct*(ifc.norbt-ifc.nocct)
    N = full.matrix((2*lwop,))
    ij = 0
    for i, j in  jwop(ifc):
        N[ij] = mat[i, j]
        N[ij+lwop] = mat[j, i]
        ij += 1
    return N

def jwop(ifc):
    """Generate  orbital excitation list"""
    for i in range(ifc.nisht):
        for j in range(ifc.nisht, ifc.nocct):
            yield (i, j)
    for i in range(ifc.nisht):
        for j in range(ifc.nocct, ifc.norbt):
            yield (i, j)
    for i in range(ifc.nisht,ifc.nocct):
        for j in range(ifc.nocct, ifc.norbt):
            yield (i, j)


if __name__ == "__main__":
    import sys
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument('prop', help='Response property')
    parser.add_argument('filename', help='File of response vectors(RSPVEC)')
    parser.add_argument('--w', type=float, default=0., help='Frequency')
    args = parser.parse_args()
    rvec = read(args.prop, propfile=args.filename, freqs=(args.w,))
    print(rvec.keys())



"""Extract molecular info from Dalton molecule file"""

ll = ('s', 'p', 'd', 'f', 'g', 'h', 'i')

def readin(inputfile):
    """Read from DALTON.BAS"""
    import sys
#
# Read inputfile into list lines
#
    molinp = open(inputfile,'r')
    lines = molinp.readlines()
#
# Check if input format is INTGRL, remove all lines before that
#
    start = lines.index('INTGRL\n')
    if start > 0: del lines[0:start]
#
# Two comment lines
#
    comments = lines[1:3]
#
# Symmetry info extracted form this line
#
    symline = lines[3]
    atom_types = int(symline[1:5])
    charge = float(symline[5:8])
#
# The molecule is a list of atom types which are hashes with structure
#    atom{'charge'} charge of this atom type
#    atom{'label'}  list of atom label strings of this type
#    atom{'center'} list of atom label strings of this type
#    atom{'basis '} structure of form
#                   [angmom][block]{'prim':[], 'cont':[][]}
#
    molecule = []
    for atype in range(atom_types):
        atom = {}
        molecule.append(atom)
        atline = lines.pop(4)
        atfield = atline.split()
#
#   Information for this atom type
#
        atom['charge'] = float(atfield[0])
        unique = int(atfield[1])
        maxl = int(atfield[2])
        atom['label'] = []
        atom['center'] = []
        atom['basis'] = []
#
#   Information for all atoms of this atom type
#
        for at in range(unique):
            line = lines.pop(4)
            field = line.split()
            atom['label'].append([])
            atom['label'][at] = field[0]
            atom['center'].append([])
            for i in range(1, 4): 
                atom['center'][at].append(float(field[i]))
#
#   Information for the basis set of this atom type
#
        for l in range(maxl):
            atom['basis'].append([])
            nbl = int(atfield[3+l])
            for block in range(nbl):
                atom['basis'][l].append([])
                atom['basis'][l][block] = {}
                atom['basis'][l][block]['prim'] = []
                atom['basis'][l][block]['cont'] = []
                line = lines.pop(4)
                field = line.split()
                npr = int(field[1])
                nco = int(field[2])
                for prim in range(npr):
                    line = lines.pop(4)
                    if nco > 3:
                        line += lines.pop(4)
                    if nco > 6:
                        line += lines.pop(4)
                    field = line.split()
                    atom['basis'][l][block]['prim'].append(float(field[0]))
                    atom['basis'][l][block]['cont'].append([])
                    for cont in range(nco):
                        atom['basis'][l][block]['cont'][prim].append(
                            float(field[cont+1])
                            )
    return molecule

def printbasis(molecule):
    """Print molecular info returned by function molecule"""
    i = 0
    for atom in molecule:
        i += 1
        print("Atom type %d charge %f" % (i, atom['charge']))
        for at in range(len(atom['label'])):
            print("center" + " %d "%(at+1) + str(atom['center'][at]))
        for l in range(len(atom['basis'])):
            print("%s-functions" % ll[l])
            for block in range(len(atom['basis'][l])):
                prim = atom['basis'][l][block]['prim']
                cont = atom['basis'][l][block]['cont']
                for ip in range(len(prim)):
                    print("    %s %s"%(prim[ip], cont[ip][:]))


#Test nuclear repulsion

def atoms_in(molecule):
    """Whats this"""
    atomlist = []
    ia = 0
    for atype in molecule:
        for a in atype["center"]:
            atomlist.append({})
            Z = atype["charge"]
            atomlist[ia]["charge"] = Z
            atomlist[ia]["center"] = a
            atomlist[ia]["type"] = atype
            contracted = 0
            for l in range(len(atype['basis'])):
                contracted_l = 0
                for block in range(len(atype['basis'][l])):
                    cont = atype['basis'][l][block]['cont']
                    contracted += len(cont[0])*(2*l+1)
                    contracted_l += len(cont[0])*(2*l+1)
                atomlist[ia][ll[l]] = contracted_l
            atomlist[ia]["contracted"] = contracted
            ia += 1
    return atomlist

def contracted_per_atom(molecule):
    """Return number of contracted per atom"""
    atomlist = atoms_in(molecule)
    contracted = []
    for atom in atomlist:
        contracted.append(atom["contracted"])
    return contracted

def contracted_per_atom_l(molecule):
    """Return number of contracted and angular momentum"""
    atomlist = atoms_in(molecule)
    contracted_l = []
    for i in range(len(atomlist)):
        atom = atomlist[i]
        contracted_l.append([])
        for l in range(len(atom["type"]["basis"])):
            contracted_l[i].append(atom[ll[l]])
    return contracted_l

def occupied_per_atom(molecule):
    """Return number of occupied per atom (assuming ANO type basis)"""
    cpal = contracted_per_atom_l(molecule)
    atomlist = atoms_in(molecule)
    assert len(cpal) == len(atomlist)
    nocclist = []
    for a in range(len(atomlist)):
        offset = 0
        atom = atomlist[a]
        Z = int(atom["charge"]+0.5)
        nocclist.append([])
        for l in range(len(cpal[a])):
            if Z <= 1:
                if l == 0:
                    nocclist[a].append(offset)
            elif Z <= 2:
                if l == 0:
                    nocclist[a].append(offset)
                    nocclist[a].append(offset+1)
            elif Z <= 8:
                if l == 0:
                    nocclist[a].append(offset)
                    nocclist[a].append(offset+1)
                elif l == 1:
                    nocclist[a].append(offset)
                    nocclist[a].append(offset+1)
                    nocclist[a].append(offset+2)
            elif Z <= 18:
                if l == 0:
                    nocclist[a].append(offset)
                    nocclist[a].append(offset+1)
                    nocclist[a].append(offset+2)
                elif l == 1:
                    nocclist[a].append(offset)
                    nocclist[a].append(offset+1)
                    nocclist[a].append(offset+2)
                    nocclist[a].append(offset+3)
                    nocclist[a].append(offset+4)
                    nocclist[a].append(offset+5)
            else:
                raise Exception( "occupied_per_atom:not implemented")
            offset += cpal[a][l]
    return nocclist

def nuc(molecule):
    """Test nuclear repulsion energy"""
    atomlist = atoms_in(molecule)
    Z = 0.0
    for a in atomlist:
        ZA = a["charge"]
        RA = a["center"]
        ia = atomlist.index(a)
        for b in atomlist[:ia]:
            ZB = b["charge"]
            RB = b["center"]
            Z += ZA*ZB/dist(RA, RB)
    return Z

def dist(A, B):
    """distance beteen two atoms"""
    from math import sqrt
    return sqrt((A[0]-B[0])**2 + (A[1]-B[1])**2 + (A[2]-B[2])**2)

if __name__ == "__main__":
    mo = readin("DALTON.BAS")
    printbasis(mo)



"""Module with blocked matrix class"""


class matrix(object):
    """Blocked matrix class"""
    def __init__(self, nrow, ncol):
        """Initialize blocked matrix instance

        nrow: integer tuple, blocked row dimension
        ncol: integer tuple, blocked column dimension
        """
        from . import full
        self.rowblocks = len(nrow)
        self.colblocks = len(ncol)
        self.nrow = nrow
        self.ncol = ncol
        self.subblock = []
        self.irow = []
        self.icol = []
        for i in range(self.rowblocks):
            self.subblock.append([])
            self.irow.append(sum(self.nrow[:i]))
            self.icol.append(sum(self.ncol[:i]))
            for j in range(self.colblocks):
                self.subblock[i].append([])
                self.subblock[i][j] = full.matrix((nrow[i], ncol[j]))

    def __str__(self):
        """String representation of blocked matrix"""

        retstr = ""
        for i in range(self.rowblocks):
            for j in range(self.colblocks):
                retstr += "\nBlock (%d,%d)\n" % (i+1, j+1) + \
                    str(self.subblock[i][j])
        return retstr

    def T(self):
        """Transpose of blocked matrix"""

        new = matrix(self.ncol, self.nrow)
        for i in range(self.rowblocks):
            for j in range(self.colblocks):
                new.subblock[i][j] = self.subblock[j][i].transpose()
        return new

    def __mul__(self, other):
        """Multiplication of blocked matrices"""

        new = matrix(self.nrow, other.ncol)
        for i in range(self.rowblocks):
            for j in range(other.colblocks):
                if self.nrow[i]*other.ncol[j]:
                    for k in range(self.colblocks):
                        new.subblock[i][j] = \
                            self.subblock[i][k] * other.subblock[k][j]
        return new

    def random(self):
        """Fill matrix with random numbers"""
        for i in range(self.rowblocks):
            for j in range(self.colblocks):
                self.subblock[i][j].random()

    def unblock(self):
        """Unblock to full matrix"""

        from . import full
        nrows = sum(self.nrow)
        ncols = sum(self.ncol)
        new = full.matrix((nrows, ncols))
        for i in range(self.rowblocks):
            for j in range(self.colblocks):
                new[
                    self.irow[i]: self.irow[i] + self.nrow[i],
                    self.icol[j]: self.icol[j] + self.ncol[j]
                    ] = self.subblock[i][j]
        return new

if __name__ == "__main__": #pragma: no cover
    pass



import unittest
import numpy as np
from math import sqrt, cos, exp, pi
from ..full import matrix, init, unit, permute

class TestMatrix(unittest.TestCase):

    def test_diag(self):
        ref = [[1, 0, 0], [0, 1, 0], [0, 0, 1]]
        this = matrix.diag([1,1,1])
        np.testing.assert_equal(this, ref)

    def test_str0(self):
        M = matrix(3)
        M[:] = range(3)
        self.assertEqual(str(M.max()), "    2.00000000")

    def test_str1(self):
        M = matrix((2,))
        M[0] = M[1] = 1
        this = str(M)
        ref = """
 (2,) 
              Column   1
       1      1.00000000
       2      1.00000000
"""
        self.assertEqual(this, ref)

    def test_str2a(self):
        M = matrix((2,2))
        M[0,0] = M[1,1] = 1
        this = str(M)
        ref = """
 (2, 2) 
              Column   1    Column   2
       1      1.00000000    0.00000000
       2      0.00000000    1.00000000
"""
        self.assertEqual(this, ref)

    def test_str2b(self):
        M = matrix((6,6))
        for i in range(6):
            M[i, i] = float(i)
        this = str(M)
        ref = """
 (6, 6) 
              Column   1    Column   2    Column   3    Column   4    Column   5
       2      0.00000000    1.00000000    0.00000000    0.00000000    0.00000000
       3      0.00000000    0.00000000    2.00000000    0.00000000    0.00000000
       4      0.00000000    0.00000000    0.00000000    3.00000000    0.00000000
       5      0.00000000    0.00000000    0.00000000    0.00000000    4.00000000

              Column   6
       6      5.00000000
"""
        self.assertEqual(this, ref)

    def test_str3(self):
        M = matrix((2,2,2))
        M[0,0,0] = M[1,1,1] = 1
        this = str(M)
        ref = """
 (2, 2, 2) 
[0]
 (2, 2) 
              Column   1    Column   2
       1      1.00000000    0.00000000
[1]
 (2, 2) 
              Column   1    Column   2
       2      0.00000000    1.00000000
"""
        self.assertEqual(this, ref)

    def test_str4(self):
        M = matrix((2,2,2,2))
        M[0,0,0,0] = M[1,1,1,1] = 1
        this = str(M)
        ref = """
 (2, 2, 2, 2) 
[0, 0]
 (2, 2) 
              Column   1    Column   2
       1      1.00000000    0.00000000
[1, 0]
 (2, 2) 
              Column   1    Column   2
[0, 1]
 (2, 2) 
              Column   1    Column   2
[1, 1]
 (2, 2) 
              Column   1    Column   2
       2      0.00000000    1.00000000
"""
        self.assertEqual(this, ref)

    def test_mul(self):
        M = matrix((2, 2))
        M[0, 0] = M[1, 1] = 2.0
        M[0, 1] = M[1, 0] = 1.0
        np.testing.assert_equal(M*M, [[5, 4], [4, 5]])

    def test_mul_scalar_matrix(self):
        M = matrix((2, 2))
        M[0, 0] = M[1, 1] = 2.0
        M[0, 1] = M[1, 0] = 1.0
        np.testing.assert_equal(2*M, [[4, 2], [2, 4]])

    def test_mul_matrix_scalar(self):
        M = matrix((2, 2))
        M[0, 0] = M[1, 1] = 2.0
        M[0, 1] = M[1, 0] = 1.0
        np.testing.assert_equal(M*2, [[4, 2], [2, 4]])

    def test_outer(self):
        V1 = matrix((2, )); V1[0]=1.0
        V2 = matrix((2, )); V2[1]=1.0
        V12 = V1.x(V2)
        np.testing.assert_equal(V12, [[0, 1], [0, 0]])

    def test_div_scalar(self):
        M = matrix((2, 2))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        np.testing.assert_equal(M/2, [[0.5, 1.0], [1.5, 2.0]])

    def test_div_self(self):
        M = matrix((2, 2))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        np.testing.assert_almost_equal(M/M, [[1.0, 0.0], [0.0, 1.0]])

    def test_neg(self):
        M = matrix((2, 2))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        np.testing.assert_equal(-M, [[-1.0, -2.0], [-3.0, -4.0]])

    def test_and(self):
        M = matrix((2, 2))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        np.testing.assert_equal(M&M, 30)

    def test_scatter(self):
        M = matrix((2, 2))
        M1 = matrix((3, 3))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        M.scatter(M1, [0, 2], [0, 2]) 
        np.testing.assert_equal(M1, [[1, 0, 2], [0, 0, 0], [3, 0, 4]])

    def test_scatter_rows(self):
        M = matrix((2, 2))
        M1 = matrix((3, 2))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        M.scatter(M1, rows=[0, 2])
        np.testing.assert_equal(M1, [[1, 2], [0, 0], [3, 4]])

    def test_scatter_columns(self):
        M = matrix((2, 2))
        M1 = matrix((2, 3))
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        M.scatter(M1, columns=[0, 2])
        np.testing.assert_equal(M1, [[1, 0, 2], [3, 0, 4]])

    def test_scatter_add(self):
        M = matrix((2, 2))
        M1 = matrix((3, 3)); M1[1, 1] = 5
        M[0, 0] = 1; M[0, 1] = 2; M[1, 0] = 3; M[1, 1] = 4
        M.scatteradd(M1, [0, 2], [0, 2]) 
        np.testing.assert_equal(M1, [[1, 0, 2], [0, 5, 0], [3, 0, 4]])

    def test_pack(self):
        M = matrix((2, 2))
        M1 = matrix((3, 3))
        M1[0, 0] = 1; M1[0, 2] = 2; M1[2, 0] = 3; M1[2, 2] = 4
        M1.packto(M, [0, 2], [0, 2]) 
        np.testing.assert_equal(M, [[1, 2], [3, 4]])

    def test_pack_rows(self):
        M = matrix((2, 3))
        M1 = matrix((3, 3))
        M1[0, 0] = 1; M1[0, 2] = 2; M1[2, 0] = 3; M1[2, 2] = 4
        M1.packto(M, rows=[0, 2])
        np.testing.assert_equal(M, [[1, 0, 2], [3, 0, 4]])

    def test_pack_columns(self):
        M = matrix((3, 2))
        M1 = matrix((3, 3))
        M1[0, 0] = 1; M1[0, 2] = 2; M1[2, 0] = 3; M1[2, 2] = 4
        M1.packto(M, columns=[0, 2])
        np.testing.assert_equal(M, [[1, 2], [0, 0], [3, 4]])

    def test_commute(self):
        z = init([[1, 0], [0, -1]])
        x = init([[0, 1], [1, 0]])
        np.testing.assert_equal(z^x, [[0, 2], [-2, 0]])

    def test_inv_init_none(self):
        M = init([[2, 1], [1, 2]])
        self.assertEqual(M._I, None)

    def test_inv(self):
        M = init([[3, 1], [1, -3]])
        M.inv()
        np.testing.assert_almost_equal(M._I, [[0.3, 0.1], [0.1, -0.3]])

    def test_rdiv(self):
        M = init([[3., 1.], [1., -3.]])
        np.testing.assert_almost_equal(1/M, [[0.3, 0.1], [0.1, -0.3]])

    def test_det(self):
        M = init([[3, 1], [1, -3]])
        self.assertAlmostEqual(M.det(), -10)

    def test_minor(self):
        M1 = matrix((3, 3))
        M1[0, 0] = 1; M1[0, 2] = 2; M1[2, 0] = 3; M1[2, 2] = 4
        np.testing.assert_equal(M1.minor(1, 1), [[1, 2], [3, 4]])

    def test_cofactor(self):
        M = init([[4, 1], [1, 3]])
        np.testing.assert_almost_equal(M.cofactor(), [[3., -1.], [-1., 4.]])

    def test_eig(self):
        M = init([[0, 1], [1, 0]])
        np.testing.assert_almost_equal(M.eig(), [-1., 1.])

    def test_eigvec(self):
        M = init([[0, 1], [1, 0]])
        np.testing.assert_almost_equal(sqrt(2)*M.eigvec()[1], [[-1., 1.],[1,1]])

    def test_qr_Q(self):
        A = init([[12, 6, -4], [-51, 167, 24], [4, -68, -41]])
        Q, R = A.qr()
        np.testing.assert_almost_equal(Q, 
            -init([[6./7, 3./7, -2./7], [-69./175, 158./175, 6./35], [-58./175, 6./175, -33./35]])
            )
    def test_qr_R(self):
        A = init([[12, 6, -4], [-51, 167, 24], [4, -68, -41]])
        Q, R = A.qr()
        np.testing.assert_almost_equal(R, 
            -init([[14, 0, 0], [21, 175, 0], [-14, -70, 35]])
            )
        np.testing.assert_almost_equal(A, Q*R)

    def test_normalize(self):
        v = init([1.0, 1.0])
        v.normalize()
        np.testing.assert_almost_equal(v, [sqrt(0.5), sqrt(0.5)])

    def test_normalize_with_overlap(self):
        v = init([1.0, 1.0])
        S = init([[1.0, 0.5], [0.5, 1.0]])
        v.normalize(S)
        np.testing.assert_almost_equal(v, [sqrt(1.0/3), sqrt(1.0/3)])

    def test_gram_schmidt(self):
        Delta = 0.1
        S = init([[1.0, Delta], [Delta, 1.0]])
        v = init([[1.0, 0.0], [0.0, 1.0]])
        u = v.GS(S)
        u_ref = init([[1.0, 0.0], [-Delta/sqrt(1-Delta**2), 1.0/sqrt(1-Delta**2)]])
        np.testing.assert_almost_equal(u, u_ref)

    def test_gram_schmidt_as_transformation(self):
        Delta = 0.1
        S = init([[1.0, Delta], [Delta, 1.0]])
        v = init([[1.0, 0.0], [0.0, 1.0]])
        T = v.GST(S)
        u = v*T
        u_ref = init([[1.0, 0.0], [-Delta/sqrt(1-Delta**2), 1.0/sqrt(1-Delta**2)]])
        np.testing.assert_almost_equal(u, u_ref)
        
    def test_sqrt(self):
        Delta = 0.1
        S = init([[1.0, Delta], [Delta, 1.0]])
        Sh = S.sqrt()
        np.testing.assert_almost_equal(Sh*Sh, S)

    def test_sqrtinv(self):
        Delta = 0.1
        S = init([[1.0, Delta], [Delta, 1.0]])
        Sih = S.invsqrt()
        np.testing.assert_almost_equal(Sih*Sih, S.I)

    def test_funcsqrt(self):
        Delta = 0.1
        S = init([[1.0, Delta], [Delta, 1.0]])
        Sh = S.func(sqrt)
        np.testing.assert_almost_equal(Sh*Sh, S)

    def test_exp(self):
        Delta = 0.1
        k = init([[Delta, 0.0], [0.0, -Delta]])
        exp_k = init([[exp(Delta), 0.0], [0.0, exp(-Delta)]])
        np.testing.assert_almost_equal(k.exp(), exp_k)

    def test_sym(self):
        A = init([[1, 2], [3, 4]])
        np.testing.assert_almost_equal(A.sym(), [[1, 2.5], [2.5, 4]])

    def test_asym(self):
        A = init([[1, 2], [3, 4]])
        np.testing.assert_almost_equal(A.antisym(), [[0, 0.5], [-0.5, 0]])

    def test_mul_triangluar(self):
        from ..full import triangular
        A = triangular.init([1, 2, 3])
        np.testing.assert_allclose(A*A, [[5, 8], [8, 13]])

    def test_scalar_mul_triangluar(self):
        from ..full import triangular
        A = triangular.init([1, 2, 3])
        np.testing.assert_allclose(2*A, [2., 4., 6.])

    def test_pack_triangular(self):
        from ..full import triangular
        A = init([[1, 2], [3, 4]])
        B = triangular.init([1, 2.5, 4])
        np.testing.assert_almost_equal(A.pack(), B)

    def test_pack_triangular_anti(self):
        from ..full import triangular
        A = init([[1, 2], [3, 4]])
        B = triangular.init([0, -0.5, 0])
        np.testing.assert_almost_equal(A.pack(anti=True), B)

    def test_lower(self):
        from ..full import triangular
        A = init([[1, 2], [3, 4]])
        B = triangular.init([1, 2, 4])
        np.testing.assert_almost_equal(A.lower(), B)

    def test_fold(self):
        from ..full import triangular
        A = init([[1, 2], [3, 4]])
        B = triangular.init([1, sqrt(2)*2, 4]) #???
        np.testing.assert_almost_equal(A.fold(), B)

    def test_norm2(self):
        A = init([3, 4])
        self.assertAlmostEqual(A.norm2(), 5.0) 

    def test_block(self):
        A = init([[1, 2], [3, 4]])
        B = A.block([1, 1], [1, 1])
        self.assertEqual(B.subblock[0], [1])
        self.assertEqual(B.subblock[1], [4])

    def test_subblocked(self):
        A = init([[1, 2], [3, 4]])
        B = A.subblocked([1, 1], [1, 1])
        self.assertEqual(B.subblock[0][1], [3])
        self.assertEqual(B.subblock[1][0], [2])

    def test_clear(self):
        A = init([[1, 2], [3, 4]])
        A.clear()
        np.testing.assert_equal(A, [[0, 0], [0, 0]])

    def test_cross(self):
        A = init([1, 2, 3])
        np.testing.assert_equal(A.cross(), [[0, -3, 2], [3, 0, -1], [-2, 1, 0]])

    def test_rot(self):
        A = init([1, 0, 0])
        z = init([0, 0, 1])
        np.testing.assert_equal(A.rot(pi/2, z), [0, 1, 0])

    def test_rot2(self):
        A = init([1, 0, 0])
        z = init([0, 0, 1])
        o = init([1, 1, 0])
        np.testing.assert_equal(A.rot(pi/2, z, origin=o), [2, 1, 0])

    def test_dist(self):
        A = init([0, 0, 1])
        np.testing.assert_almost_equal(A. dist([0, 1, 0]), sqrt(2))

    def test_angle3(self):
        A = init([2., 0., 0.])
        B = init([1., 0., 0.])
        C = init([1., 1., 0.])
        self.assertAlmostEqual(A.angle3(B, C), pi/2)
    
    def test_angle3d(self):
        A = init([2., 0., 0.])
        B = init([1., 0., 0.])
        C = init([1., 1., 0.])
        self.assertAlmostEqual(A.angle3d(B, C), 90.)

    def test_angle(self):
        A = init([1., 0., 0.])
        B = init([1., 1., 0.])
        np.testing.assert_almost_equal(A.angle(B), pi/4)

    def test_angle2(self):
        A = init([.1, .0, 0.])
        B = init([-.1, .1, 0.])
        np.testing.assert_almost_equal(A.angle(B), 3*pi/4)

    def test_angled(self):
        A = init([0, 0, 1])
        B = init([0, 1, 0])
        np.testing.assert_almost_equal(A.angled(B), 90)


    def test_dihedral_open(self):
        A = init([1, 1, 0])
        B = init([1, 0, 0])
        C = init([0, 0, 0])
        D = init([0, -1, 0])
        np.testing.assert_equal(A.dihedral(B, C, D), pi)

    def test_dihedrald_open(self):
        A = init([1, 1, 0])
        B = init([1, 0, 0])
        C = init([0, 0, 0])
        D = init([0, -1, 0])
        np.testing.assert_equal(A.dihedrald(B, C, D), 180)

    def test_dihedral_eclipsed(self):
        A = init([1, 1, 0])
        B = init([1, 0, 0])
        C = init([0, 0, 0])
        D = init([0, 1, 0])
        np.testing.assert_equal(A.dihedral(B, C, D), 0)

    def test_svd(self):
        A = init([[1, 1, sqrt(3)], [-1, -1, 0]])
        u, s, v = A.svd()
        np.testing.assert_almost_equal(A, u*s*v.T)

    def test_unit(self):
        I = init([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
        np.testing.assert_almost_equal(unit(3), I)

    def test_unit2(self):
        I = init([[2, 0, 0], [0, 2, 0], [0, 0, 2]])
        np.testing.assert_almost_equal(unit(3, factor=2), I)

    def test_permute(self):
        np.testing.assert_equal(permute([0, 2], 4), 
            [[1, 0, 0, 0],
             [0, 0, 1, 0],
             [0, 1, 0, 0],
             [0, 0, 0, 1],])


if __name__ == "__main__": # pragma: no cover
    unittest.main()



import unittest
import numpy

class LoPropTestCase(unittest.TestCase):

    def assert_str(self, this, ref):
        def stripm0(numstr):
            # allow string inequality from round-off errors
            return numstr.replace("-0.000", " 0.000")
        thism0 = stripm0(this)
        refm0 = stripm0(ref)
        self.assertEqual(thism0, refm0)


    def assert_allclose(self, *args, **kwargs):
        kwargs['atol'] = kwargs.get('atol', 1e-5)
        if 'text' in kwargs:
            kwargs['err_msg'] = kwargs.get('text', '')
            del(kwargs['text'])
        numpy.testing.assert_allclose(*args, **kwargs)



#!/usr/bin/env python
"""Module for reading data from Dalton one-electron integral file AOONEINT"""
import sys 
import struct
import numpy as np
from .util import unformatted, full, blocked

def readhead(filename="AOONEINT"):
    """Read data in header of AOONEINT"""
    aooneint = unformatted.FortranBinary(filename)
    aooneint.next()
    if len(aooneint.data) == 144:
        #
        # Newer versions: title in own record
        #
        title = aooneint.data
        #
        # Next record contains MAXREP...
        #
        aooneint.next()
    else:
        #
        # Older versions: title ans MAXREP in same record
        #
        title = "".join(aooneint.readbuf(192,'c'))
    #
    # Buffer should now contains MAXREP data
    #

    FLOAT = 'd'
    INT = _get_integer_format(aooneint.rec)

    nsym = aooneint.readbuf(1, INT)[0]
    naos = aooneint.readbuf(nsym, INT)
    potnuc = aooneint.readbuf(1, FLOAT)[0]
    unlabeled = {
        "ttitle": title,
        "nsym": nsym,
        "naos": naos,
        "potnuc": potnuc,
        "int_fmt": INT,
        "float_fmt": FLOAT
        }
    aooneint.close()
    return unlabeled

def _get_integer_format(header_record):
    """Determine integer format from first record of AOONTINT"""
    # from herrdn.F:
    # WRITE (LUONEL) MAXREP+1,(NAOS(I),I=1,MAXREP+1), POTNUC,
    #&              (0.D0,I=1,2) ! so record is minimum 32 bytes
    floatsize = 8
    dimensions = np.array(
        [[intsize*(nsym+1) + 3*floatsize for intsize in [4, 8]
         ] for nsym in [1, 2, 4, 8]]
        )
    if header_record.reclen in dimensions[:, 0]:
       integer_format = 'i'
    elif header_record.reclen in dimensions[:, 1]:
       integer_format = 'q'
    else:
       raise Exception("Unknown binary format in AOONEINT")

    return integer_format


def readisordk(filename="AOONEINT"):
    """Read data under label ISORDK in AOONEINT"""

    header = readhead(filename)
    INT = header['int_fmt']
    FLOAT = header['float_fmt']

    aooneint = unformatted.FortranBinary(filename)
    table1 = aooneint.find("ISORDK")
    aooneint.next()
    aooneint.next()
    sizeofi = struct.calcsize(INT)
    sizeofd = struct.calcsize(FLOAT)
    mxcent_ = (len(aooneint.data)-sizeofi)//(4*sizeofd)
    chrn_ = aooneint.readbuf(mxcent_, FLOAT)
    nucdep = aooneint.readbuf(1, INT)[0]
    cooo_ = aooneint.readbuf(3*mxcent_, FLOAT)
    isordk_ = {
        "table":table1,
        "chrn":chrn_,
        "nucdep":nucdep,
        "cooo":cooo_,
        }
    aooneint.close()
    return isordk_

def readscfinp(filename="AOONEINT"):
    """Read data under labl SCFINP in AOONEINT"""

    header = readhead(filename)
    INT = header['int_fmt']
    FLOAT = header['float_fmt']

    aooneint = unformatted.FortranBinary(filename)
    table2 = aooneint.find("SCFINP")
    scfinp_ = {}
    scfinp_["table"] = table2
    aooneint.next()
    if len(aooneint.data) == 144:
        title = aooneint.data
        aooneint.next()
    else:
        title = aooneint.readbuf(192,'c')
    scfinp_["ttitle"] = title
    nsym = aooneint.readbuf(1, INT)[0]
    scfinp_["nsym"] = nsym
    scfinp_["naos"] = aooneint.readbuf(nsym, INT)
    scfinp_["potnuc"] = aooneint.readbuf(1, FLOAT)[0]
    kmax = aooneint.readbuf(1, INT)[0]
    scfinp_["kmax"] = kmax
    scfinp_["ncent"] = aooneint.readbuf(kmax,INT)
    nbasis = aooneint.readbuf(1,INT)[0]
    scfinp_["nbasis"] = nbasis
    scfinp_["jtran"] = aooneint.readbuf(nbasis,INT)
    scfinp_["itran"] = aooneint.readbuf(8*nbasis,INT)
    scfinp_["ctran"] = aooneint.readbuf(8*nbasis,FLOAT)
    scfinp_["nbasis"] = aooneint.readbuf(1,INT)[0]
    scfinp_["inamn"] = aooneint.readbuf(nbasis,INT)
    scfinp_["iptyp"] = aooneint.readbuf(nbasis,INT)
    scfinp_["dpnuc"] = aooneint.readbuf(3,FLOAT)
    nucdep = aooneint.readbuf(1,INT)[0]
    scfinp_["nucdep"] = nucdep
    scfinp_["cooo"] = aooneint.readbuf(3*nucdep,FLOAT)
    scfinp_["ifxyz"] = aooneint.readbuf(3,INT)
    scfinp_["dummy"] = aooneint.readbuf(1,FLOAT)[0]
    scfinp_["qpol"] = aooneint.readbuf(6,FLOAT)
    scfinp_["qq"] = aooneint.readbuf(3,FLOAT)
    scfinp_["jfxyz"] = aooneint.readbuf(3,INT)
    aooneint.close()
    return scfinp_

def read(label="OVERLAP", filename="AOONEINT"):
    """Read integral for label"""
    lbuf = 600
    #
    # Initialize
    #
    unlabeled = readhead(filename)
    nsym = unlabeled["nsym"]
    nbas = unlabeled["naos"]
    INT = unlabeled["int_fmt"]
    FLOAT = unlabeled["float_fmt"]
    nnbast = 0
    for nbasi in nbas:
        nnbast += nbasi*(nbasi+1)//2
    s = full.matrix((nnbast,))
    #
    # Open file, locate label
    #
    aooneint = unformatted.FortranBinary(filename)
    labinfo = aooneint.find(label)
    #
    # Loop over records
    #

    for rec in aooneint:
       buf = rec.read(lbuf, FLOAT)
       ibuf = rec.read(lbuf, INT)
       length, = rec.read(1, INT)
       if length < 0: break
       for i, b in zip(ibuf[:length], buf[:length]):
           s[i-1] = b

    _S = blocked.triangular(nbas)
    off = 0
    for isym in range(nsym):
        nbasi = nbas[isym]*(nbas[isym]+1)//2
        _S.subblock[isym] = np.array(s[off:off+nbasi]).view(full.triangular)
        off += nbasi
    #aooneint.close()
    return _S


if __name__ == "__main__":
    import argparse
    from util.timing import timing

    parser = argparse.ArgumentParser()
    
    parser.add_argument('-H', '--head', dest='head', action='store_true')
    parser.add_argument('-i', '--isordk', dest='isordk', action='store_true')
    parser.add_argument('-s', '--scfinp', dest='scfinp', action='store_true')
    parser.add_argument('-v', '--verbose', dest='verbose', action='store_true')
    parser.add_argument('-u', '--unpack', dest='unpack', action='store_true')
    parser.add_argument('-l', '--label')
    parser.add_argument('aooneint')
    
    args = parser.parse_args()

    if args.head: 
        t = timing('head')
        head = readhead(args.aooneint)
        print("Header on AOONEINT")
        for k in head:
            print(k, head[k])
        print(t)

    if args.isordk:
        t = timing('getisrordk')
        isordk = readisordk(args.aooneint)
        print("isordk table", isordk["table"])
        n = isordk["nucdep"]
        chrn = isordk["chrn"]
        cooo = isordk["cooo"]
        mxcent = len(chrn)
        print("nucdep=%i" % n, "mxcent=%i" % mxcent)
        print(full.init(chrn)[:n])
        print(full.init(cooo).reshape((3, mxcent))[:, :n])
        print(t)

    if args.scfinp:
        t = timing('scfinp')
        scfinp = readscfinp(args.aooneint)
        for k in scfinp:
            print(k, scfinp[k])
        print(t)

    if args.label is not None:
        tread = timing("read")
        s1 = read(label=args.label, filename=args.aooneint)
        print(tread)
        if args.unpack:
            t = timing("unpack")
            s2 = s1.unpack()
            print(t)
            t = timing("unblock")
            S = s2.unblock()
            print(t)
        else:
            S = s1
        if args.verbose:
            print(args.label, S)



"""
    This module defines a class FortranBinary for interaction with binary files 
    generated by FORTRAN unformatted I/O
"""
import struct, sys

class FortranBinary():
    """Class for binary files compatible with Fortran Unformatted I/O"""
    pad = 4
    def __init__(self, name, status=None):
        self.name = name
        if status == 'new':
            self.file = open(name,'wb')
        else:
            try:
                self.file = open(name, 'rb', 10)
            except(IOError):
                print("\n%s: file %s not found\n" % (__name__ + '.' + self.__class__.__name__, name))
                raise IOError
        self.data = None
        self.loc = 0
        self.reclen = 0

    def __iter__(self):
        return self

    def __next__(self): #pragma: no cover
        return self.next()

    def next(self):
        """Read a Fortran record"""
        head = self.file.read(self.pad)
        if head:
            size = struct.unpack('i', head)[0]
            self.data = self.file.read(size)
            self.reclen = size
            tail = self.file.read(self.pad)
            assert head == tail
            self.loc = 0
            self.rec = Rec(self.data) 
            return self.rec
        else:
            raise StopIteration

    def readbuf(self, n, c):
        """Read data from current record"""
        vec = self.rec.read(n, c)
        return vec

    def find(self, label):
        """Find string label in file"""
        if type(label) == str:
            try:
                blabel = bytes(label, 'utf-8')
            except TypeError:
                blabel = label
        elif type(label) == bytes:
            blabel = label
        else:
            raise ValueError

        for rec in self:
            if blabel in rec: return rec

    def close(self):
        """Close file"""
        self.file.close()

    def record_byte_lengths(self):
        reclengths = [self.reclen for record in self]
        return tuple(reclengths)

class Rec():
    def __init__(self, data):
        self.data = data
        self.loc = 0
        self.reclen = len(data)

    def __contains__(self, obj):
        return obj in self.data

    def read(self, n, c):
        """Read data from current record"""
        start, stop = self.loc, self.loc+struct.calcsize(c*n)
        vec = struct.unpack(c*n, self.data[start:stop])
        self.loc = stop
        return vec
        
if __name__ == "__main__": #pragma: no cover
    pass



import unittest
import numpy as np
from ..full import triangular

class TestTriangular(unittest.TestCase):

    def setUp(self):
        self.twodim = triangular.init([1.,  2.,  3.])
        np.random.seed(0)
        self.antisym = triangular((2, 2), anti=True)
        self.antisym[1, 0] = 2.0


    def test_init_square_shape(self):
        self.assertEqual(self.twodim.sshape, (2, 2))

    def test_init_values(self):
        np.testing.assert_equal(self.twodim, [1., 2., 3.])

    def test_str(self):
        self.assertEqual(str(self.twodim), """
    1.00000000
    2.00000000    3.00000000
"""
        )

    def test_getitem_by_tuple(self):
        self.assertEqual(self.twodim[0, 0], 1.)
        self.assertEqual(self.twodim[1, 0], 2.)
        self.assertEqual(self.twodim[0, 1], 2.)
        self.assertEqual(self.twodim[1, 1], 3.)

    def test_getitem_by_tuple_as(self):
        self.assertEqual(self.antisym[0, 0], 0.)
        self.assertEqual(self.antisym[1, 0], 2.)
        self.assertEqual(self.antisym[0, 1], -2.)
        self.assertEqual(self.antisym[1, 1], 0.)

    def test_getitem_by_scalar(self):
        self.assertEqual(self.twodim[0], 1.)
        self.assertEqual(self.twodim[1], 2.)
        self.assertEqual(self.twodim[2], 3.)

    def test_setitem(self):
        self.twodim[0, 1] = 4.
        np.testing.assert_equal(self.twodim, [1., 4., 3.])

    def test_setitem_anti1(self):
        self.antisym[0, 1] = -2.
        np.testing.assert_equal(self.antisym, [0., 2., 0.])

    def test_setitem_anti2(self):
        self.antisym[1, 0] = 2.
        np.testing.assert_equal(self.antisym, [0., 2., 0.])

    def test_unpack(self):
        np.testing.assert_equal(self.twodim.unpack(), [[1., 2.], [2., 3.]])

    def test_unpack_as(self):
        As = triangular((2, 2), anti=True)
        As.random()
        np.testing.assert_allclose(As.unpack(), [[0.0000000,  -0.71518937],  [0.71518937, 0.00000000]])

    def test_mul(self):
        np.testing.assert_equal(self.twodim*self.twodim, [[5., 8.], [8., 13.]])

    def test_mul_scalar(self):
        np.testing.assert_equal(self.twodim*2, [2., 4., 6])

    def test_random(self):
        self.twodim.random()
        np.testing.assert_allclose(self.twodim, [ 0.5488135 ,  0.71518937,  0.60276338])

    def test_random_antisymmetric(self):
        As = triangular((2, 2), anti=True)
        As.random()
        np.testing.assert_allclose(As, [ 0.0000000 ,  0.71518937,  0.00000000])

if __name__ == "__main__": #pragma: no cover
    unittest.main()



import unittest
import os 
import numpy as np
from ..core import penalty_function, xtang, pairs, MolFrag
from ..daltools.util import full

import re
thisdir  = os.path.dirname(__file__)
case = "ch4_absorption"
tmpdir=os.path.join(thisdir, case, 'tmp')
exec('from . import %s_data as ref'%case)


import unittest


class NewTest(unittest.TestCase):

    def setUp(self):
    # modify Gagliardi penalty function to include unit conversion bug
        self.m = MolFrag(tmpdir, freqs=(0.4425,), damping=0.004556, pf=penalty_function(2.0/xtang**2))

    def tearDown(self):
        pass

    def test_nuclear_charge(self):
        Z = self.m.Z
        np.testing.assert_allclose(Z, ref.Z)

    def test_coordinates_au(self):
        R = self.m.R
        np.testing.assert_allclose(R, ref.R)

    def test_default_gauge(self):
        np.testing.assert_allclose(ref.Rc, self.m.Rc)

    def test_total_charge(self):
        Qtot = self.m.Qab.sum()
        np.testing.assert_allclose(Qtot, ref.Qtot)

    def test_total_dipole(self):
        np.testing.assert_allclose(self.m.Dtot, ref.Dtot, atol=1e-6)

    def test_total_charge_shift_real(self):
        self.m.set_real_pol()
        RedQ = self.m.dQa[0].sum(axis=0).view(full.matrix)
        np.testing.assert_allclose(RedQ, [0,0,0], atol=1e-5)

    def test_total_charge_shift_imag(self):
        self.m.set_imag_pol()
        ImdQ = self.m.dQa[0].sum(axis=0).view(full.matrix)
        np.testing.assert_allclose(ImdQ, [0,0,0], atol=1e-5)

    def test_real_polarizability_total(self):

        self.m.set_real_pol()
        ref_Am = full.init([
            [30.854533, -0.000004,  0.000000],
            [-0.000004, 30.854527,  0.000000],    
            [ 0.000000,  0.000000, 30.854522]
            ])
        Am = self.m.Am[0]
        np.testing.assert_allclose(Am, ref_Am, rtol=1e-6, atol=1e-6)

    def test_imag_polarizability_total(self):

        self.m.set_imag_pol()
        ref_Am = full.init([
            [1.228334, 0.000000, 0.000000],
            [0.000000, 1.228335, 0.000000],    
            [0.000000, 0.000000, 1.228333]])
        Am = self.m.Am[0]
        np.testing.assert_allclose(Am, ref_Am, rtol=1e-6, atol=1e-6)

    def test_capture_error_setup(self):
        self.m._real_pol = False
        self.m._imag_pol = False
        with self.assertRaises(ValueError):
            Dk = self.m.Dk

if __name__ == "__main__": #pragma: no cover
    unittest.main()



#!/usr/bin/env python
"""Module for reading data fro property integral file AOPROPER"""
import os
import math
import numpy as np
from .util import unformatted, full
from . import one, sirifc, dens, rspvec

def read(*args, **kwargs):
    """Read property integral"""
    propfile = kwargs.get("filename")
    if not propfile:
        tmpdir = kwargs.get('tmpdir', '/tmp')
        propfile = os.path.join(tmpdir, 'AOPROPER')

    unpack = kwargs.get("unpack", True)
    AOPROPER = unformatted.FortranBinary(propfile)
    mat = {}
    for rec in AOPROPER:
        buf = rec.read(32, 'c')
        #stars, data, symtype, label = (
        #      buf[:8], buf[8:16], buf[16:24], buf[24:32]
        #      )
        stars = b"".join(buf[:8])
        symtype = b"".join(buf[16:24])
        blabel = b"".join(buf[24:32]).strip()

        bargs = (s.encode() for s in args)
        if blabel.strip() in bargs:
            label = blabel.decode()
            rec = next(AOPROPER)
            buffer_ = rec.read(rec.reclen//8, 'd')
            if symtype == b"SQUARE  ":
                n = int(round(math.sqrt(matsize)))
                assert n*n == matsize
                mat[label] = full.init(buffer_).reshape((n, n))
            else:
                mat[label] = np.array(buffer_).view(full.triangular)
                mat[label].anti = (symtype == b"ANTISYMM")
    if unpack:
        return tuple([mat[lab].unpack() for lab in args])
    else:
        return tuple([mat[lab] for lab in args])

def grad(*args, **kwargs):
    tmpdir = kwargs.get('tmpdir', '/tmp')

    propmat = read(*args, **kwargs)

    AOONEINT = os.path.join(tmpdir, 'AOONEINT')
    S = one.read(label='OVERLAP', filename=AOONEINT).unpack().unblock()

    SIRIFC  = os.path.join(tmpdir, 'SIRIFC')
    ifc = sirifc.sirifc(SIRIFC)
    Da, Db = dens.Dab(ifc_=ifc)
    cmo = ifc.cmo.unblock()
    cmoS = cmo*S
        
    G = (rspvec.tovec(
         cmo.T*(S*Da*P.T - P.T*Da*S)*cmo
       + cmo.T*(S*Db*P.T - P.T*Db*S)*cmo,
        ifc
        )
      for P in propmat)

    return tuple(G)

if __name__ == "__main__":
    import sys, getopt
    from util.timing import timing
    t_all = timing("main")
    unpack = False
    verbose = False
    tmpdir = '.'
    try:
        opt, arg = getopt.getopt(
            sys.argv[1:],'t:uv',['tmpdir', 'unpack','verbose']
            )
        for o, v in opt:
            if o in ('-u','--unpack'):
                unpack = True
            if o in ('-v','--verbose'):
                verbose = True
            if o in ('-t','--tmpdir'):
                tmpdir = v
        proplabel = arg[0]
    except IndexError:
        print("Usage: %s property" % sys.argv[0])
        sys.exit(1)
    t_read = timing("prop.read")
    aoproper = os.path.join(tmpdir, 'AOPROPER')
    propint = read(prop=proplabel, propfile=aoproper)
    print(t_read)
    if unpack:
        t_unpack = timing("unpack")
        propsq = propint.unpack()
        print(t_unpack)
    if verbose:
        if unpack:
            print(propsq)
        else:
            print(propint)
    print(t_all)



import time
class timing:
   def __init__(self,info):
      self.info=info
      self.t0=time.clock()
      self.w0=time.time()
      self.t1=None
      self.w1=None
   def __str__(self):
      if self.t1 is None: self.stop()
      t=self.t1-self.t0
      w=self.w1-self.w0
      return "Time used in %-20s:%10.2f (cpu) %10.2f (wall)"""%(self.info,t,w)
   def stop(self):
      self.t1=time.clock()
      self.w1=time.time()



#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup
import os

try:
    from pip.req import parse_requirements
except ImportError:
    raise SystemExit("ERROR: No pip installed")

#if submodules have not been initiated, just update them
os.system( 'git submodule update --init --recursive' )

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements( os.path.join( os.getcwd(), 'requirements.txt') , session=False)

# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
reqs = [str(ir.req) for ir in install_reqs]

setup(name="LoProp",
    version="0.1.7",
    packages = find_packages(),
    #package_dir = { '' : 'loprop' },
    scripts = ['scripts/loprop'],
    package_data = { 'loprop.test' : [ '*/tmp/RSPVEC',
                                       '*/tmp/DALTON.BAS',
                                       '*/tmp/AOONEINT',
                                       '*/tmp/SIRIFC',
                                       '*/tmp/AOPROPER', ],
                     'loprop.daltools.test' : ['data/*', 
                         'test*/RSPVEC',
                         'test*/DALTON.BAS',
                         'test*/AOONEINT',
                         'test*/SIRIFC',
                         'test*/AOPROPER',
                         'test*/E3VEC',
                         ],
                     'loprop.daltools.util.test' : [ 'fort.1', 'fort.2', 'fort.3'], 
                     },
    exclude_package_data = { 'loprop.test': ['*/tmp/MOTWOINT'] },
    #scripts=["loprop/loprop.py",],
    author="Olav Vahtras",
    author_email="vahtras@kth.se",
    install_requires = install_reqs,
    description = 'LoProp implementation for Dalton',
    )



"""Matrix utility module based on numpy"""

import math
import numpy
from . import subblocked, blocked

class matrix(numpy.ndarray):
    """ A subclass of numpy.ndarray for matrix syntax and better printing """
    fmt = "%14.8f"
    order = 'F'

    def __new__(cls, shape, fmt=None):
        """Constructor ..."""            
        obj = numpy.zeros(shape, order=cls.order).view(cls)
        return obj

    def __array_finalize__(self, obj):
        if obj is None: return
        self._I = None

    def __str__(self):
        """Output formatting of matrix object, inspired by Dalton OUTPUT

        Example:
        >>> M=matrix((2,2)); M[0,0]=M[1,1]=1; print M
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      1.00000000    0.00000000
               2      0.00000000    1.00000000
        <BLANKLINE>
        """
     
        retstr = '\n %s \n' % str(self.shape)
        if len(self.shape) == 1:
            r = self.shape[0]
            columnsperblock = 1
            fullblocks = 1
            trailblock = 0
            for b in range(fullblocks):
                crange = range(b*columnsperblock,(b+1)*columnsperblock)
                retstr += " "*10
                for j in crange:
                    retstr += "    Column%4d" % (j+1)
                retstr += '\n'
                for i in range(r):
                    rownorm = math.fabs(self[i])
                    if rownorm > 1e-8:
                        retstr += "%8d  " % (i+1)
                        retstr = retstr + self.fmt % self[i]
                        retstr += '\n'
        elif len(self.shape) == 2:
            r, c = self.shape
            if 0:
                pass
            else:
                columnsperblock = 5
                fullblocks = c//columnsperblock
                trailblock = c % 5
                for b in range(fullblocks):
                    crange = range(b*columnsperblock,(b+1)*columnsperblock)
                    retstr += " "*10
                    for j in crange:
                        retstr += "    Column%4d" % (j+1)
                    retstr += '\n'
                    for i in range(r):
                        rownorm = self[i, crange].norm2()
                        if rownorm > 1e-8:
                            retstr += "%8d  " % (i+1)
                            for j in crange:
                                retstr = retstr + self.fmt % self[i, j]
                            retstr += '\n'
                    retstr += '\n'
                crange = range(fullblocks*columnsperblock, c)
                if trailblock:
                    retstr += " "*10
                    for j in crange:
                        retstr += "    Column%4d" % (j+1)
                    retstr += '\n'
                    for i in range(r):
                        rownorm = self[i, crange].norm2()
                        if rownorm > 1e-8:
                            retstr += "%8d  " % (i+1)
                            for j in crange:
                                retstr += self.fmt % self[i, j]
                            retstr += '\n'
        elif len(self.shape) > 2:
            r, c = self.shape[:2]
            hishape = self.shape[2:]
            losize = r*c
            hisize = self.size//losize
            altshape = (r, c, hisize)
            alt = self.reshape(altshape, order=matrix.order)
            #
            # Given linear index n find tuple idx=(i0,i1,i2...) 
            # for actual shape hishape=(n0,n1,n2..)
            #
            for n in range(hisize):
                k = n
                idx = []
                for i in range(len(hishape), 1, -1):
                    #
                    # integer divide by n0*...n(i-2), save remainder
                    #
                    dp = numpy.asarray(hishape[:i-1]).prod()
                    idx.append(k//dp)
                    k = k % dp
                idx.append(k)
                idx.reverse()
             
                retstr += str(idx) + str(alt[:, :, n])
        elif len(self.shape) == 0: #returned by numpy.max
            return self.fmt % self.sum()
        return retstr

    def __mul__(self, other):
        """Matrix multiplication
        Example:
        >>> M = matrix((2,2)); M[0,0]=M[1,1]=2.0; M[0,1]=M[1,0]=1.0
        >>> print M*M
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      5.00000000    4.00000000
               2      4.00000000    5.00000000
        <BLANKLINE>
     
        >>> print 2*M
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      4.00000000    2.00000000
               2      2.00000000    4.00000000
        <BLANKLINE>
     
        >>> print M*2
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      4.00000000    2.00000000
               2      2.00000000    4.00000000
        <BLANKLINE>
        """
     
        if isinstance(other, self.__class__) or self.is_sibling(other):
            return numpy.dot(self, other)
        else:
            return other*self

    def is_sibling(self, other):
        return self.__class__.__mro__[1] == other.__class__.__mro__[1]
 
    def x(self, other):
        """Outer product
        Example: (1,0).T*(0,1)
        >>> v1=matrix((2,))
        >>> v2=matrix((2,))
        >>> v1[0]=v2[1]=1.0
        >>> print v1
        <BLANKLINE>
         (2,) 
                      Column   1
               1      1.00000000
        <BLANKLINE>
        <BLANKLINE>
     
        >>> print v2
        <BLANKLINE>
         (2,) 
                      Column   1
               2      1.00000000
        <BLANKLINE>
        <BLANKLINE>
     
        >>> print v1.x(v2)
        <BLANKLINE>
         (2, 2) 
                      Column   1    Column   2
               1      0.00000000    1.00000000
        <BLANKLINE>
        """
     
        c = numpy.outer(self, other).reshape(self.shape+other.shape)
        return c.view(matrix)
     
    def __truediv__(self, other):
        """Solution of linear equation/inversion
        Example:
        >>> A=matrix((2,2)).random(); x=matrix((2,1)).random()
        >>> b=A*x
        >>> print x-b/A
        <BLANKLINE>
         (2, 1) 
                      Column   1
        <BLANKLINE>
        """
     
        if isinstance(other, self.__class__) or self.is_sibling(other):
            new = numpy.linalg.solve(other, self)
        else:
            new = (1.0/other)*self
        return new

    def __div__(self, other):
        return self.__truediv__(other)

    def scatteradd(self, other, rows=None, columns=None):
        """ See scatter"""
        self.scatter(other, rows, columns, add=1)

    def scatter(self, other, rows=None, columns=None, add=0):
        """
        Usage: scatter copies element into other matrix
        with indices other[rows[k],columns[l]]=self[k, l]
        """
        if not add:
            other.clear()
        r, c = self.shape
        if rows and columns:
            assert(r == len(rows))
            assert(c == len(columns))
            for i in range(r):
                for j in range(c):
                    other[rows[i], columns[j]] += self[i, j]
        else:
            if rows:
                assert(r == len(rows))
                for i in range(r):
                    other[rows[i], :] += self[i, :]
            if columns:
                assert(c == len(columns))
                for j in range(c):
                    other[:, columns[j]] += self[:, j]
        return

    def packto(self, other, rows=None, columns=None, add=0):
        """
        Usage pack copies element into other matrix
        with indices other[k, columns[l]=self[rows[k], columns[l]
        """
        other_rdim, other_cdim = other.shape
        if not add:
            other.clear()
        if rows and columns:
            assert(other_rdim == len(rows))
            assert(other_cdim == len(columns))
            for i in range(other_rdim):
                for j in range(other_cdim):
                    other[i, j] += self[rows[i], columns[j]]
        else:
            if rows:
                assert(other_rdim == len(rows))
                for i in range(other_rdim):
                    other[i, :] += self[rows[i], :]
            if columns:
                assert(other_cdim == len(columns))
                for j in range(other_cdim):
                    other[:, j] += self[:, columns[j]]
        return


    def __and__(self, other):
        """Shortcut for sum(ij) A_{ij} B_{ij}"""
        return numpy.dot(self.ravel(self.order), other.ravel(other.order))

    def __xor__(self, other):
        """Commutator [A, B]"""
        return self*other - other*self

    #@property
    def inv(self):
        """Matrix inverse"""
        if self._I is None:
            r, c = self.shape
            assert r == c
            self._I = unit(r)/self
        return self._I
    I = property(fget=inv)

    def __rtruediv__(self, other):
        """Division with Matrix instance in denominator"""
        r, _ = self.shape
        return unit(r, other)/self

    def __rdiv__(self, other):
        return self.__rtruediv__(other)

    def tr(self):
        """Trace"""
        return self.trace()

    def det(self):
        """Return determinant"""
        return numpy.linalg.det(self)

    def minor(self, i, j):
        """Matrix minor"""
        r, c = self.shape
        rows = list(range(r))
        cols = list(range(c))
        rows.remove(i)
        cols.remove(j)
        # bug
        #print self[rows, cols]
        # do rows and cols separately
        redr = self[rows, :]
        redc = redr[:, cols]
        return redc

    def cofactor(self):
        """Co-factor matrix"""
        r, c = self.shape
        assert(r == c)
        new = matrix((r, c))
        for i in range(r):
            for j in range(c):
                new[i, j] = (-1)**(i+j)*self.minor(i, j).det()
        return new

    def eig(self):
        """
        Return sorted eigenvalues as a column matrix
        """
        eigvals = numpy.linalg.eigvals(self)
        p = eigvals.argsort()
        return eigvals[p].view(matrix)

    def eigvec(self):
        """
        Return eigenvalue/eigenvector pair, sorted
        by eigenvalue
        """
        U, V = numpy.linalg.eig(self)
        p = U.argsort()
        #
        # Note that eig returns U as ndarray but V as its subclass
        #
        return U[p].view(matrix), V[:, p]

    def qr(self):
        """
        Return eigenvalue/eigenvector pair, sorted
        by eigenvalue
        """
        Q, R = numpy.linalg.qr(self)
        return Q, R

    def normalize(self, S=None):
        """Normalizing myself"""
        if S is None:
            norm = 1/math.sqrt(self&self)
        else:
            norm = 1/math.sqrt(self&(S*self))
        self *= norm

    def GS(self, S, T=False):
        """Gram-Schmidt orthonormalization"""
        r, c = self.shape
        new = matrix((r, c))
        new[:, 0] = self[:, 0]
        new[:, 0].normalize(S)
        for i in range(1, c):
            P = new[:, :i]*new[:, :i].T*S
            new[:, i] = self[:, i] - P*self[:, i]
            new[:, i].normalize(S)
        #
        # relation new=self*T
        #          self.T*new=self.T*self*T
        #          (self.T*self).inv()*self.T*new = T (QR?)
        #
        if T:
            return (new.T*S*self).inv()
        else:
            return new

    def GST(self, S):
        """Gram-Schmidt orthonormalization, return transformation matrix"""
        return self.GS(S, T=True)
          
    def sqrt(self):
        """Return square root of matrix"""
        from scipy.linalg import sqrtm
        return sqrtm(self).real.view(matrix)

    def invsqrt(self):
        """Return inverse square root of matrix"""
        from scipy.linalg import sqrtm
        return sqrtm(self.I).real.view(matrix)

    @staticmethod
    def diag(vec):
        """Make diagonal matrix from vector"""
        return numpy.diag(vec).view(matrix)

    def func(self, f):
        """General function of matrix"""
        val, vec = numpy.linalg.eig(self)
        fval = val.view(matrix)
        n = len(val)
        new = matrix((n, n))
        for i in range(n):
            fval[i] = f(val[i])
            new[i, i] = fval[i]
        return vec*new*vec.inv()


    def exp(self):
        """Exponential of matrix"""
        r, _ = self.shape
        new = unit(r)
        termnorm = new&new
        term = new*1
        i = 0
        while (termnorm > 1e-8):
            i = i+1
            term *= self/i
            new += term
            termnorm = math.sqrt(term&term)
        return new
           
    def random(self):
        """Fill myself with random numbers"""
        self.flat = numpy.random.random(self.size)
        return self

    def sym(self):
        """return symmetrized matrix"""
        return .5*(self + self.T)

    def antisym(self):
        """return anti-symmetrized matrix"""
        return .5*(self - self.T)

    def pack(self, anti=False):
        """Pack to triangular"""
        #raise Exception("test")
        r, c = self.shape
        assert r == c
        _t = triangular(self.shape)
        fac = 1
        if anti: 
            fac = -1
        for i in range(r):
            for j in range(i+1):
                _t[i, j] = .5*(self[i, j] + fac*self[j, i])
        return _t

    def lower(self):
        """Return lower triangular"""
        r, c = self.shape
        assert r == c
        _t = triangular(self.shape)
        for i in range(r):
            for j in range(i+1):
                _t[i, j] = self[i, j]
        return _t

    def fold(self):
        """Return 'folded' matrix"""
        r, c = self.shape
        assert r == c
        _t = triangular(self.shape)
        for i in range(c):
            for j in range(i):
                _t[i, j] = math.sqrt(2)*self[i, j]
            _t[i, i] = self[i, i]
        return _t

    def norm2(self):
        """Euclidean norm"""
        return numpy.linalg.norm(self)

    def block(self, rdim, cdim):
        """Return blocked version of matrix
        Only diagonal blocks defined by input dimensions are considered
        """
        assert len(rdim) == len(cdim)
        new = blocked.BlockDiagonalMatrix(rdim, cdim)
        rstart = 0
        cstart = 0
        for i in range(len(rdim)):
            new.subblock[i] = self[rstart:rstart+rdim[i], cstart:cstart+cdim[i]]
            rstart += rdim[i]
            cstart += cdim[i]
        return new

    def subblocked(self, rdim, cdim):
        """Return fully blocked version of matrix
        """
        new = subblocked.matrix(rdim, cdim)
        rstart = 0
        for i in range(new.rowblocks):
            cstart = 0
            for j in range(new.colblocks):
                new.subblock[i][j] = \
                    self[rstart:rstart+rdim[i], cstart:cstart+cdim[j]]
                cstart += cdim[j]
            rstart += rdim[i]
        return new

    def clear(self):
        """Zero myself"""
        self[...] = 0

    def cross(self, *args):
        """ With out argument return 3x3 matrix Ax, 
            with a vector return ordinary cross product AxB"""
        assert self.shape == (3,)
        if not args:
            new = matrix((3, 3))
            new[0, 1] = -self[2]
            new[1, 0] = self[2]
            new[0, 2] = self[1]
            new[2, 0] = -self[1]
            new[1, 2] = -self[0]
            new[2, 1] = self[0]
            return new
        else:
            c = matrix(3)
            a = self
            b, = args
            c[0] = a[1]*b[2] - a[2]*b[1]
            c[1] = a[2]*b[0] - a[0]*b[2]
            c[2] = a[0]*b[1] - a[1]*b[0]
            return c

    def dist(self, other):
        """Distance between two points"""
        return (self - other).norm2()
 
    def angle3(self, B, C):
        """Return A-B-C angle"""
        return (self - B).angle(C-B)

    def angle3d(self,  B, C):
        """Return A-B-C angle in degrees"""
        return self.angle3(B, C)*180/math.pi

    def angle(self, other):
        """Return A-O-B angle, O origin"""
        dot = self & other
        cos2a = dot*dot/((self&self)*(other&other))
        if dot > 0:
            cosa = math.sqrt(cos2a)
        else:
            cosa = -math.sqrt(cos2a)
        return math.acos(cosa)

    def angled(self, other):
        """Return A-O-B angle in degrees, O origin"""
        return self.angle(other)*180/math.pi

    def rot(self, angle, vec, origin=None):
        """Rotate self by an angle around vec"""

        p = vec[:]/vec.norm2()
        if origin is None:
            so = self
        else:
            so = self - origin
         
        sp = p*(p&so)
        sq = so - sp
        sr = p.cross(sq)
        #print "pqr", p, q, r
        self[:] = sp + sq*math.cos(angle) + sr*math.sin(angle)
        if origin is not None:
            self[:] += origin
        return self

    def dihedral(self, r3, r2, r1):
        """Return dihedral angle A-B-C-D"""
        b3 = self-r3
        b2 = r3-r2
        b1 = r2-r1
        b1xb2 = b1.cross()*b2
        b2xb3 = b2.cross()*b3
        n2 = b2.norm2()
        return math.atan2(
            n2*b1&b2xb3,
            b1xb2&b2xb3
            )

    def dihedrald(self, r3, r2, r1):
        """Return dihedral angle A-B-C-D in degrees"""
        return self.dihedral(r3, r2, r1)*180/math.pi

    def svd(self):
        """Compact singular value decomposition
        input n, p
        output u(n, p)
               s(p, p)
               v(p, p)
        """
        u, s, vt = numpy.linalg.svd(self, full_matrices=0)
        s = numpy.diag(s).view(matrix)
        return  u, s, vt.T

    def sum(self, **kwargs):
        """Was previously handled by numpy.sum but 
           as of ubuntu 12.04 numpy.sum returns matrix([sum]) rather than sum
        """
        return numpy.sum(self.view(numpy.ndarray), **kwargs)

def unit(n, factor=1):
    """Return unit matrix, optionally scaled"""
    vec = matrix((n*n,))
    vec[:n*n:n+1] = factor
    return vec.reshape((n, n))

def permute(select, n):
    """Return permutation matrix based on input"""
    complement = list(range(n))
    for i in range(n):
        if i in select:
            complement.remove(i)
    permlist = select+complement
    new = matrix((n, n))
    for i in range(n):
        new[permlist[i], i] = 1
    return new

def init(nestlist):
    """Create and initialize matrix object"""
    return numpy.array(nestlist).view(matrix).T
#
# init should be generalized with matrix.order, now 'F' assumed -> traspose
#

#class triangular(numpy.ndarray):
class triangular(matrix):
    """Triangular packed matrix class"""

    def __new__(cls, shape, anti=False, fmt=None):
        tshape = ((shape[0]*(shape[0]+1))/2,)
        obj = matrix(tshape).view(cls)
        obj.sshape = shape
        obj.anti = anti
        if fmt is None:
            obj.fmt = matrix.fmt
        return obj

    def __array_finalize__(self, obj):
        if obj is None: return
        self.dim = int(math.sqrt(0.25+2*obj.size))
        self.sshape = getattr(obj, 'sshape', (self.dim, self.dim))
        self.anti = getattr(obj, 'anti', False)
        self.fmt = getattr(obj, 'fmt', matrix.fmt)
       
    @staticmethod
    def init(arr):
        """initialize with array"""
        n = int(round(-0.5 + math.sqrt(0.25+2*len(arr))))
        # should test for valid n
        new = triangular((n, n))
        ij = 0

        for i in range(n):
            for j in range(i+1):
                new[i, j] = arr[ij]
                ij += 1
        return new



    def __str__(self):
        """Output triangular matrix (Dalton OUTPAK)"""
        retstr = "\n"
        r, _ = self.sshape
        for i in range(r):
            for j in range(i+1):
                retstr += self.fmt % (self[i, j])
            retstr += "\n"
        return retstr

    def __getitem__(self, args):
        vec = self.view(matrix)
        if type(args) == int:
            return vec[args]
        else:
            i, j = args
            if self.anti and i < j:
                ij = j*(j+1)/2+i
                return -vec[ij]
            else:
                ij = i*(i+1)/2+j
                return vec[ij]

    def __setitem__(self, args, value):
        vec = self.view(matrix)
        i, j = args
        if i < j and self.anti:
            ij = j*(j+1)/2+i
            vec[ij] = -value
        else:
            ij = i*(i+1)/2+j
            vec[ij] = value

    def unpack(self):
        """Unpack from triangular to square"""
        n = self.sshape[0]
        new = matrix((n, n))
        for i in range(n):
            new[i, i] = self[i, i]
            for j in range(i):
                new[i, j] = self[i, j]
                if self.anti:
                    new[j, i] = -self[i, j]
                else:
                    new[j, i] = self[i, j]
        return new

    def __mul__(self, other):
        """Multiplication of triangular matrices, return square"""
        if isinstance(other, self.__class__):
            return self.unpack()*other.unpack()
        else:
            return other*self

    def random(self):
        """Triangular random matrix"""
        matrix.random(self)
        if self.anti:
            for i in range(self.sshape[0]):
                self[i, i] = 0
        return self




import unittest
try:
    import mock
except ImportError: #pragma: no cover
    from unittest import mock

from . import timing


class TimeTest(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    @mock.patch('time.clock')
    @mock.patch('time.time')
    def test_set(self, mock_time, mock_clock):
        r0 = timing.timing('set')
        r0.stop()
        self.assertEqual(mock_time.call_count, 2)
        self.assertEqual(mock_clock.call_count, 2)

    @mock.patch('time.clock')
    @mock.patch('time.time')
    def test_get(self, mock_time, mock_clock):
        mock_time.side_effect = [0, 0.15]
        mock_clock.side_effect = [0, 0.25]
        r0 = timing.timing('dummy')
        info = str(r0)
        self.assertEqual(info, """\
Time used in dummy               :      0.25 (cpu)       0.15 (wall)"""
            )

    
        

if __name__ == "__main__": #pragma: no cover
    unittest.main()



#!/usr/bin/env python
"""Quadratic response module"""

import os
from .sirifc import sirifc
from .dens import ifc as dens_ifc
from .prop import read as prop_read
from .rspvec import read as rspvec_read
from .rspvec import tomat as rspvec_tomat
from .one import read as one_read

def QR(A, B, C, wb=0.0, wc=0.0, tmpdir='/tmp', **kwargs):
    """Calculate the linear response function <<A;B>> from response vector
    on RSPVEC for B, as <[kB,A]>"""
    ifcfile = os.path.join(tmpdir, 'SIRIFC')
    propfile = os.path.join(tmpdir, 'AOPROPER')
    ifc = sirifc(ifcfile)
    a, = prop_read(A, filename=propfile, unpack=True)
    BC = B.ljust(8) + C.ljust(8)
    #pdb.set_trace()
    dBC =  D2k(BC, bcfreqs=((wb,wc),), ifc=ifc, tmpdir=tmpdir, **kwargs)
    return a&dBC[BC,wb, wc]

def D2k(*args, **kwargs):
    """Calculated second-order density matrix for quadratic response"""
    
    bclabs = args

    bcfreqs = kwargs.get('bcfreqs', ((0.0, 0.0),))
    tmpdir = kwargs.get('tmpdir', '/tmp')
    ifc = kwargs.get('ifc', None)

    SIRIFC = os.path.join(tmpdir, 'SIRIFC')
    RSPVEC = os.path.join(tmpdir, 'RSPVEC')
    E3VEC = os.path.join(tmpdir, 'E3VEC')
    AOONEINT = os.path.join(tmpdir, 'AOONEINT')
    #
    # Read interface data from SIRIFC if not provided
    #
    if ifc is None:
        ifc = sirifc(name=SIRIFC)
    #
    # Get densities in AO basis
    #
    dc, do = dens_ifc(SIRIFC, ifc)
    d = dc+do
    #
    # Get linear response vector (no symmetry)
    #
    kzywop = 2*ifc.nwopt

    bfreqs = {wbc[0] for wbc in bcfreqs}
    cfreqs = {wbc[1] for wbc in bcfreqs}
    blabs = {bclab[:8] for bclab in bclabs}
    clabs = {bclab[8:] for bclab in bclabs}
    bkeys = [(l,w) for l in blabs for w in bfreqs]
    ckeys = [(l,w) for l in clabs for w in cfreqs]
    bckeys = [(l,wb,wc) for l in bclabs for wb in bfreqs for wc in cfreqs]


    NB = rspvec_read(
        *blabs, freqs=bfreqs,
        propfile=RSPVEC
        )
    NC = rspvec_read(
        *clabs, freqs=cfreqs,
        propfile=RSPVEC
        )
    NBC = rspvec_read(
        *bclabs, bfreqs=bfreqs, cfreqs=cfreqs,
        propfile=RSPVEC
        )
    # transform density over unique pairs
    cmo = ifc.cmo.unblock()
    kb = { lw:cmo*rspvec_tomat(NB[lw], ifc, tmpdir=tmpdir).T*cmo.T  for lw in bkeys }
    kc = { lw:cmo*rspvec_tomat(NC[lw], ifc, tmpdir=tmpdir).T*cmo.T  for lw in ckeys }

    kbc = { lw:cmo*rspvec_tomat(NBC[lw], ifc, tmpdir=tmpdir).T*cmo.T  for lw in bckeys }

    S = one_read(filename=AOONEINT).unpack().unblock()
    Sd = S*d
    dkb = { lw: kb[lw]*Sd - Sd.T*kb[lw] for lw in bkeys }
    dkc = { lw: kc[lw]*Sd - Sd.T*kc[lw] for lw in ckeys }
    dkbc = { lw: kbc[lw]*Sd - Sd.T*kbc[lw] for lw in bckeys }

    for lbc, wb, wc in bckeys:
        lb, lc = lbc[:8], lbc[8:]
        _dkbc = dkbc[(lbc, wb, wc)]
        _dkb = dkb[(lb, wb)]
        _dkc = dkc[(lc, wc)]
        _kb = kb[(lb, wb)]
        _kc = kc[(lc, wc)]
    
        _da2bc = 0.5*(
            _kc*S*_dkb - _dkb*S*_kc +
            _kb*S*_dkc - _dkc*S*_kb
            )

        if 'a2test' in kwargs:
            _dkbc.clear()
        _dkbc += _da2bc

    #Symmetrize keys
        dkbc.update({(lbc[8:] + lbc[:8], wc, wb): _dkbc})
        
    
            

    return dkbc
    
    
    
    

if __name__ == "__main__":
    import sys
    from optparse import OptionParser
    op = OptionParser()
    op.add_option(
          '-t','--tmpdir',
          dest='tmpdir', default='/tmp',
          help='scratch directory [/tmp]'
          )
    try:
        opt, arg = op.parse_args(sys.argv[1:])
        Aop, Bop, Cop = arg
        tmpdir = opt.tmpdir
    except (IndexError, ValueError):
        print("Usage: %s A B C" % sys.argv[0])
        sys.exit(1)

    print(QR(Aop, Bop, Cop, tmpdir=tmpdir))



import unittest
import os
import numpy as np
from nose.tools import raises
from .. import rspvec, sirifc

class TestRspVec(unittest.TestCase):

    def setUp(self):
        n, e = os.path.splitext(__file__)
        self.tmpdir = n + ".d"
        self.RSPVEC = os.path.join(self.tmpdir, 'RSPVEC')
        self.SIRIFC = os.path.join(self.tmpdir, 'SIRIFC')
        self.E3VEC = os.path.join(self.tmpdir, 'E3VEC')
        self.ifc = sirifc.sirifc(name=self.SIRIFC)

    def test_read(self):
        Nx = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        this = Nx[12]
        ref = -0.75732690
        self.assertAlmostEqual(this, ref)

    def test_read_w(self):
        Nx = rspvec.read("XDIPLEN", freqs=(0.5,), propfile=self.RSPVEC)[("XDIPLEN", 0.5)]
        this = Nx[12]
        ref = -2.242435
        self.assertAlmostEqual(this, ref)

    def test_read_e3(self):
        Nx = rspvec.read("XDIPLEN XDIPLEN", propfile=self.E3VEC)["XDIPLEN XDIPLEN"]
        this = Nx[1]
        ref = 0.03874785
        self.assertAlmostEqual(this, ref)

    def test_read_e3_w(self):
        Nx = rspvec.read("XDIPLEN XDIPLEN", freqs=(0.5,), propfile=self.E3VEC)[("XDIPLEN XDIPLEN",0.5)]
        this = Nx[1]
        ref = 0.05668560
        self.assertAlmostEqual(this, ref)

    @raises(rspvec.RspVecError)
    def test_read_missing(self):
        Nx = rspvec.read("WRONGLAB", propfile=self.RSPVEC)

    def test_read_all(self):
        N1, N2 = rspvec.readall("XDIPLEN", self.RSPVEC)[:4:2]
        f1, f2 = N1[1], N2[1]
        self.assertListEqual([f1, f2], [0, 0.5])
        this = (N1[0][12], N2[0][12])
        ref = (-0.75732690, -2.242435)
        np.testing.assert_almost_equal(this, ref)

    def test_tomat(self):
        Nx = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        kx = rspvec.tomat(Nx, self.ifc, tmpdir=self.tmpdir)
        this = kx[8, 3]
        ref = 0.75732690
        self.assertAlmostEqual(this, ref)

    def test_tovec(self):
        Nx = rspvec.read("XDIPLEN", propfile=self.RSPVEC)["XDIPLEN"]
        kx = rspvec.tomat(Nx, self.ifc, tmpdir=self.tmpdir)
        Nx = rspvec.tovec(kx, self.ifc, tmpdir=self.tmpdir)
        this = Nx[44]
        ref = 0.75732690
        self.assertAlmostEqual(this, ref)

    def test_jwop(self):
        this = list(rspvec.jwop(self.ifc))
        ref = [ (i, j) for i in range(8) for j in range(8, 12)]
        self.assertListEqual(this, ref)

if __name__ == "__main__":
    unittest.main()



import unittest
import os
import numpy as np
from ..util import full
from .. import dens

class TestDens(unittest.TestCase):

    def setUp(self):
        n, _ = os.path.splitext(__file__)
        self.tmpdir = n + ".d"

    def test_h1diag(self):
        diref = full.init([
        [0.5370962 ,  0.33768834,  0.        ],
        [ 0.33768834,  0.21231469,  0.        ],
        [ 0.        ,  0.        ,  0.        ]
	])


        di, da = dens.h1diag(1, 1, filename=os.path.join(self.tmpdir, "AOONEINT"))
        np.testing.assert_almost_equal(di, diref)

    def test_Dab(self):
        na = 2; nb = 1
        C = full.unit(3)
        da, db = dens.C2Dab(C, na, nb)
        daref = full.init(
            [[1., 0., 0], 
            [0., 1., 0.], 
            [0., 0., 0.]]
            )
        dbref = full.init(
            [[1., 0., 0], 
            [0., 0., 0.], 
            [0., 0., 0.]]
            )
        np.testing.assert_almost_equal(da, daref)
        np.testing.assert_almost_equal(db, dbref)
    

if __name__ == "__main__":
    unittest.main()
